/* -*- Mode: IDL; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Mozilla browser.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications, Inc.
 * Portions created by the Initial Developer are Copyright (C) 1999
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "nsISupports.idl"

interface nsIChannel;
interface nsIContentViewer;
interface nsIStreamListener;
interface nsIDocument;
interface nsILoadGroup;
interface nsIPrincipal;

/**
 * To get a component that implements nsIDocumentLoaderFactory
 * for a given mimetype, use nsICategoryManager to find an entry
 * with the mimetype as its name in the category "Gecko-Content-Viewers".
 * The value of the entry is the contractid of the component.
 * The component is a service, so use GetService, not CreateInstance to get it.
 */

[scriptable, uuid(5e7d2967-5a07-444f-95d5-25b533252d38)]
interface nsIDocumentLoaderFactory : nsISupports {
    nsIContentViewer createInstance(in string aCommand,
                              in nsIChannel aChannel,
                              in nsILoadGroup aLoadGroup,
                              in string aContentType, 
                              in nsISupports aContainer,
                              in nsISupports aExtraInfo,
                              out nsIStreamListener aDocListenerResult);

    nsIContentViewer createInstanceForDocument(in nsISupports aContainer,
                                         in nsIDocument aDocument,
                                         in string aCommand);

    /**
     * Create a blank document using the given loadgroup and given
     * principal.  aPrincipal is allowed to be null, in which case the
     * new document will get the about:blank codebase principal.
     */
    nsIDocument createBlankDocument(in nsILoadGroup aLoadGroup,
                                    in nsIPrincipal aPrincipal);
};
